# libabaqus_main.py

from abaqus import *
from abaqusConstants import *
from libabaqus_particle import *
from libabaqus_indenter import *


def setAxisymmParticleModel(
	jId=1, jobTitle="default",			
	particleType="Ellipsoid",	# particle type: 
								# 	Ellipsoid, Triangular
	pgdl=[10.,10],				# particle geometry dimension list

								# particle:  
	pEE=72,						# 	Young's modulus;defaults to fused quartz
	pvv=0.17,					# 	Poisson ratio;defaults to fused quartz

								# indenter:
	iEE=200,					#	Young's modulus 
	ivv=0.3,					#	Poisson ratio
	plasticity="PERFECT",		# plasticity type
								#	PERFECT
								#   USER (linear or exponential)
	SYield=1.5,					# Yield strength 
	hardStrength=10.,			# hardening strength
	hardExponent=0.5,			# hardening exponent
	igd=50.,					# indenter dimension

	frictionCoeff=0.,			# indenter-particle contact property 

	initialInc=1e-2,			# stepping parameters
	minInc=1e-5,
	maxInc=1e-2,
	maxNumInc=100000,

	mnes=0.1,					# max mesh element size
	mxes=0.2,					# max mesh element size
	ges=0.5,	
	iElmType="cax8",			# element type for indenter:
								# cax4 (linear) or cax8 (quadratic)

	QQ2=-1,						# prescribed BC
):
	"""
	2D static analysis of 1/4 particle subjected to 
	vertical compression by deformable punch.
	Note:
		Unit for lengths is 1e-6*m, (i.e. micrometer).
		Unit for forces is 1e-3*N, (i.e. miliNewton). 
		Unit for stress is in 1e9*Pa (i.e. GPa)
	"""

	import part
	import material
	import section
	import assembly
	import step
	import interaction
	import load
	import mesh
	import job
	import os
	import sys
	import regionToolset

	myModel=mdb.Model(name="AxisymmParticle")

###
### PARTS
###

# particle part

	myParticlePart=myModel.Part(name="particlePart",
		dimensionality=AXISYMMETRIC,type=DEFORMABLE_BODY)

	if particleType=="Ellipsoid":
		gAxisymmEllipsoid(myModel,pgdl,mnes,mxes,ges)
	if particleType=="Triangular":
		gAxisymmTriangular(myModel,pgdl,mnes,mxes,ges)

# punch part	

	myIndenterPart=myModel.Part(name="indenterPart",
			dimensionality=AXISYMMETRIC,type=DEFORMABLE_BODY)
	gAxisymmFlatPunch(myModel,pgdl,igd,mnes,mxes,ges,iElmType)


#####	
##### MATERIAL
#####

	particleMat=myModel.Material(name="elastic")
	particleMat.Elastic(type=ISOTROPIC,table=((pEE,pvv),))

	indenterMat=myModel.Material(name="plastic")
	indenterMat.Elastic(type=ISOTROPIC,table=((iEE,ivv),))
	if plasticity=="PERFECT":
		indenterMat.Plastic(hardening=ISOTROPIC,table=((SYield,0.),))
	if plasticity=="USER":
		indenterMat.Plastic(hardening=USER,
			table=((SYield,),(hardStrength,),(hardExponent,),)
		)

#####
##### SECTION	
#####

	myParticleSection=myModel.HomogeneousSolidSection(name="particleSection",
		material="elastic",thickness=1.0)
	myParticlePart.SectionAssignment(region=(myParticlePart.faces,),
		sectionName="particleSection")

	myIndenterSection=myModel.HomogeneousSolidSection(name="indenterSection",
		material="plastic",thickness=1.0)
	myIndenterPart.SectionAssignment(region=(myIndenterPart.faces,),
		sectionName="indenterSection")

####
#### ASSEMBLY
####

	myAssembly=myModel.rootAssembly

	myParticleInstance=myAssembly.Instance(name="particleInstance",
		part=myParticlePart,dependent=ON)

	myIndenterInstance=myAssembly.Instance(
		name="indenterInstance",part=myIndenterPart,dependent=ON)
	myIndenterInstance.translate(vector=(0,pgdl[1],0))

# assembly sets

	RP=myAssembly.ReferencePoint(point=(0,0,0))
	QQ=myAssembly.Set(name="QQ",
		referencePoints=(myAssembly.referencePoints[RP.id],)
	)


####
#### STEP
####

	myModel.StaticStep(name="statics0",previous="Initial",
		timePeriod=1.0, initialInc=initialInc,
		minInc=minInc,maxInc=maxInc,maxNumInc=maxNumInc,
		nlgeom=True,
	)

#	myModel.steps['statics0'].control.setValues(
#		allowPropagation=OFF, resetDefaultValues=OFF, 
#		displacementField=(1e-4,0.05,0,0,0,0,0,0,0,0,0),
#		timeIncrementation=(0,0,0,0,0,0,0,100,0,0,0),
#	)

####
#### INTERACTIONS
####

	myContactPrp=myModel.ContactProperty(name="ParticleIndenterContactPrp")
	if frictionCoeff=="inf":
		myContactPrp.TangentialBehavior(formulation=ROUGH)
	else:
		if frictionCoeff==0.: 
			myContactPrp.TangentialBehavior(formulation=FRICTIONLESS)
		else: 
			myContactPrp.TangentialBehavior(formulation=PENALTY, 
				directionality=ISOTROPIC,
				table=((frictionCoeff,),),
				fraction=0.005
			)

	myContactPrp.NormalBehavior(pressureOverclosure=HARD,
		constraintEnforcementMethod=PENALTY,	
	)

	myModel.SurfaceToSurfaceContactStd(name="ParticleIndenterContact",
		createStepName="statics0",
		master=myIndenterInstance.surfaces["CONTACT"],
		slave=myParticleInstance.surfaces["CONTACT"],
		sliding=FINITE, 
##		sliding=SMALL, 
		interactionProperty="ParticleIndenterContactPrp",
		contactTracking=ONE_CONFIG,
	)

####
#### OUTPUT
####

# fields
	
	myModel.fieldOutputRequests["F-Output-1"].suppress()

	myModel.FieldOutputRequest(name="FIELDS",
		createStepName="statics0",
		variables=("S","LE","PE","U","PEEQ","CDISP","CSTRESS")
	)

# history
# H-Output-1 contains all energies
# myModel.historyOutputRequests["H-Output-1"].suppress()

# reaction force RF2 for sets QQ and LOAD

	myModel.HistoryOutputRequest(name="QQ",
		createStepName="statics0",
		region=myAssembly.sets["QQ"],variables=("RF2","U2")
	)

# top and bottom node U2 displacement

	myModel.HistoryOutputRequest(name="BOTTOMNODE",
		createStepName="statics0",
		region=myIndenterInstance.sets["BOTTOMNODE"],
		variables=("U2",)
	)


# contact Area and Pressure

	myModel.HistoryOutputRequest(name="CAREA",
		createStepName="statics0",
		region=myParticleInstance.sets['CONTACT'],
		variables=("CAREA",)
	)

	myModel.HistoryOutputRequest(name="PCSTRESS",
		createStepName="statics0",
		region=myParticleInstance.sets['CONTACT'],
		variables=("CSTRESS","COORD")
	)

	myModel.HistoryOutputRequest(name="ICSTRESS",
		createStepName="statics0",
		region=myIndenterInstance.sets['CONTACT'],
		variables=("CSTRESS","COORD")
	)


####
#### LOAD
####

	myModel.DisplacementBC(name="sym-x",createStepName="Initial",
		region=myParticleInstance.sets["XAXIS"],u2=0.,)

	myModel.DisplacementBC(name="sym-y-particle",createStepName="Initial",
		region=myParticleInstance.sets["YAXIS"],u1=0.,)

	myModel.DisplacementBC(name="sym-y-indenter",createStepName="Initial",
		region=myIndenterInstance.sets["YAXIS"],u1=0.,)

##	myModel.DisplacementBC(name="fixed_node",createStepName="statics0",
##		region=myParticleInstance.sets["FIXED_NODE"],u1=0.)


# prescribed BC

#	myModel.DisplacementBC(name="load",createStepName="statics0",
#		region=myIndenterInstance.sets["LOAD"],u2=QQ2)

	myModel.Equation(name="QQ2",
		terms=((1.,'indenterInstance.LOAD',2),(-1.,'QQ',2),)
	)
	myModel.DisplacementBC(name="load",createStepName="statics0",
		region=QQ,u1=0.,u2=QQ2,ur3=0.)

####
#### JOB	
####
	jobName=jobTitle+"-"+str(jId)
	myJob=mdb.Job(name=jobName,model="AxisymmParticle")
	
	jobPath="jobs/"+jobName
	sys.__stderr__.write("\n "+jobPath)
	sys.__stderr__.write("\n Setting .inp file ... ")
	cwd=os.getcwd()
	if os.path.isdir(jobPath)==False:
		os.makedirs(jobPath)
		os.chdir(jobPath)
		myJob.writeInput()
		os.chdir(cwd)
		sys.__stderr__.write("OK.\n")
	else:
		if os.path.exists(jobPath+"/"+jobName+".inp")==False:
			os.chdir(jobPath)
			myJob.writeInput()
			os.chdir(cwd)
			sys.__stderr__.write("OK.\n")
		else:
			sys.__stderr__.write("Job .inp file exists! SKIPPED!\n")




