# libabaqus_python.py

from abaqusConstants import *

###
###
###
def reportSP1(
		jobName,
		stepName="statics0",
		setNames=["OUTEREDGE"],
		frames=[-1]
):
	"""
	The maximal principal stress of nodes in set from Field Requests.
	Nodal values are averaged over elements that share the node of a set.
	""" 
	
	import odbAccess
	import os 

	jobOdb="jobs/"+jobName+"/"+jobName+".odb"
	myOdb=odbAccess.openOdb(path=jobOdb)
	
	myStep=myOdb.steps[stepName]

	if frames==[]:
		frames=range(1,len(myStep.frames))
		print "(reportSP1) Processing ALL frames!"

	for setName in setNames:
		jobDataPath="data/"+jobName+"/"
		if os.path.isdir(jobDataPath)==False:
			os.makedirs(jobDataPath)
		fileName=jobDataPath+jobName+"_"+setName+"-"+stepName+"_SP1.txt"
		f=open(fileName,"w")
		print "(reportSP1) Writing file ... ", fileName
		f.write("# File automatically generated by reportSP1.\n")
		f.write("# Nodal maximal principal stress for set name "+setName+".\n")
		f.write("# All stresses are in units of 1e9Pa.\n")
		f.write("# Data format per line:\n")
		f.write("# % 6s % 8s % 13s % 13s % 13s % 13s\n" % 
			("frame", "node", "X coord", "Y coord", "Z coord", "SP1"))
		for frame in frames:
			print "(reportSP1) \tframe ... ", frame
			myFrame=myStep.frames[frame]
			myRegions=myStep.historyRegions
			
			stress=myFrame.fieldOutputs["S"]
			stress_nodal=stress.getSubset(
				region=myOdb.rootAssembly.elementSets[" ALL ELEMENTS"],
				position=ELEMENT_NODAL)
			stress_set=stress_nodal.getSubset(
				region=myOdb.rootAssembly.nodeSets[setName])
	
			SP1={}
			for n in myOdb.rootAssembly.nodeSets[setName].nodes[0]:
				sumSP1=0.
				count=0.
				for v in stress_set.values:
					if v.nodeLabel==n.label:
						count+=1.
						sumSP1+=v.maxPrincipal
				SP1[n.label]=sumSP1/count
				f.write("% 8i % 8i % 12e % 12e % 12e % 12e\n" %
					(frame,n.label,n.coordinates[0],n.coordinates[1],
					n.coordinates[2],SP1[n.label]))
		f.close()
###
###
###
def reportMaxPrincipal(
		odbFile,setName,
		setType='part',
		instanceName="PARTICLEINSTANCE",
		field='S',
		value="maxPrinc",
		stepName="statics0",
		frames=[-1]
):
	"""
	Principal value for nodes in set from Field Requests. Nodal 
	values are averaged over elements that share the node of a set.
	""" 
	
	import odbAccess
	import os 


	if (setType=='part') & (instanceName==''):
		print "(reportPrincipal) ERROR:",
		print "If setType==part instanceName must be given as argument!"
		return 


	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")

	myOdb=odbAccess.openOdb(path=odbFile)
	
	myStep=myOdb.steps[stepName]

	if frames==[]:
		frames=range(1,len(myStep.frames))
		print "(reportPrincipal) Processing ALL frames!"

	jobDataPath="data/"+jobName+"/"
	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)

	fileName=jobDataPath+jobName+"_"+setName+"-"+stepName+"_"+field+"-"+value
	fileName=fileName+".txt"
	f=open(fileName,"w")

	print "(reportPrincipal) Writing file ... ", fileName

	f.write("# File automatically generated by reportPrincipal.\n")
	f.write("# Field parameter is "+field+".\n")
	f.write("# Nodal principal value for set name "+setName+".\n")
	f.write("# Data format per line:\n")
	f.write("# % 6s % 8s % 13s % 13s % 13s % 13s\n" % 
		("frame", "node", "X coord", "Y coord", "Z coord", field+"."+value))

	for frame in frames:

		print "(reportPrincipal) \tframe ... ", frame
	
		myFrame=myStep.frames[frame]
		stress=myFrame.fieldOutputs[field]
		stress_nodal=stress.getSubset(
			region=myOdb.rootAssembly.elementSets[" ALL ELEMENTS"],
			position=ELEMENT_NODAL)
	
		if setType=='root':
			setRegion=myOdb.rootAssembly.nodeSets[setName]
			setNodes=setRegion.nodes[0]

		if setType=='part':
			setRegion=myOdb.rootAssembly.instances[
				instanceName].nodeSets[setName]
			setNodes=setRegion.nodes

		stress_set=stress_nodal.getSubset(region=setRegion)
		PV={}
		for n in setRegion.nodes:
			sumPV=0.
			count=0.
			for v in stress_set.values:
				if v.nodeLabel==n.label:
					count+=1.
					if value=="maxPrinc":
						sumPV+=v.maxPrincipal
					if value=="maxInPlane":
						sumPV+=v.maxInPlanePrincipal
			PV[n.label]=sumPV/count
			f.write("% 8i % 8i % 12e % 12e % 12e % 12e\n" %
				(frame,n.label,n.coordinates[0],n.coordinates[1],
				n.coordinates[2],PV[n.label]))

	f.close()

###
###
###
def reportForceDisplacement(
	odbFile,
	parameter="",
	setType="root",
	stepName="",
	instanceName="ASSEMBLY",
	setName="QQ",
):
	"""
	Force or displacement component from history request for all nodes in set.
	"""	

	import odbAccess
	import os
	from numpy import zeros

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]

	nodeData={}

	if setType=="root":
		nSet=myOdb.rootAssembly.nodeSets[setName].nodes[0]
	if setType=="part":
		nSet=myOdb.rootAssembly.instances[instanceName].nodeSets[setName].nodes
	for n in nSet:
		nKey="Node "+instanceName+"."+str(n.label)
		nData=myStep.historyRegions[nKey].historyOutputs[parameter].data
		nodeData[n.label]=nData

	fileName=jobDataPath+"/"+jobName+"_"+setName+"-"+stepName+"_"+parameter+".hro"
	print "(reportForceDisplacement) Writing ...", fileName
	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	f=open(fileName,"w")

	f.write("# Data retrieved by reportForceDisplacement.\n")
	f.write("# History output for parameter "+parameter+" of nodes in "+setName+".\n")
	f.write("# Displacements are in 1e-6m, and forces in 1e-3N.\n")
	f.write("# Data format per line: nodeID, value_frame-1, value_frame-2, ... , value-last-frame\n")
	
	for nKey in nodeData.keys():
		line="% 5s " % str(nKey)
		for d in nodeData[nKey]:
			line=line+"% 13s " % (str(d[1]))
		f.write(line+"\n")
	f.close()

###
###
###
def reportCArea(
	odbFile,
	stepName="statics0",
):
	"""
	Reporting contact area values.
	"""

	import odbAccess
	import os
	from numpy import zeros

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]

	nKey=myStep.historyRegions["NodeSet CONTACT"].historyOutputs.keys()[0]
	nData=myStep.historyRegions["NodeSet CONTACT"].historyOutputs[nKey].data

	fileName=jobDataPath+"/"+jobName+"_CAREA.hro"
	print "(reportCArea) Writing ...", fileName
	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	f=open(fileName,"w")

	f.write("# Data retrieved by reportCArea.\n")
	f.write("# History output for parameter CAREA.\n")
	f.write("# Area is in um^2.\n")
	f.write("# Data format per line: value_frame-1, value_frame-2, ... , value-last-frame\n")

	line=""	
	for d in nData:
		line=line+"% 13s " % (str(d[1]))
	f.write(line+"\n")
	f.close()


###
###
###
def reportCPress(
	odbFile,
	stepName="statics0",
	setName="CONTACT",
	instanceName="PARTICLEINSTANCE",
):
	"""
	Reporting contact pressure for nodes in a set from history report.
	"""

	import odbAccess
	import os
	from numpy import zeros

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]
	
	cpress_par="CPRESS   ASSEMBLY_PARTICLEINSTANCE_CONTACT/ASSEMBLY_INDENTERINSTANCE_CONTACT"

	nodeData={}
	nodeCoords={}

	nSet=myOdb.rootAssembly.instances[instanceName].nodeSets[setName].nodes
	for n in nSet:
		nKey="Node "+instanceName+"."+str(n.label)
		nData=myStep.historyRegions[nKey].historyOutputs[cpress_par].data
		nCoordX=myStep.historyRegions[nKey].historyOutputs["COOR1"].data
		nCoordY=myStep.historyRegions[nKey].historyOutputs["COOR2"].data
		nodeData[n.label]=nData
		nodeCoords[n.label]=[nCoordX,nCoordY]

	numberFrames=len(nData)

	if instanceName=="PARTICLEINSTANCE":
		fileName=jobDataPath+"/"+jobName+"_CPRESSP.hro"
	if instanceName=="INDENTERINSTANCE":
		fileName=jobDataPath+"/"+jobName+"_CPRESSI.hro"

	print "(reportCPress) Writing ...", fileName
	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	f=open(fileName,"w")

	f.write("# Data retrieved by reportCPress.\n")
	f.write("# History output for parameter CPRESS.\n")
	f.write("# Contact pressure is in GPa.\n")
	f.write("# Data format per line:\n")
	f.write("# % 6s % 8s % 13s % 13s % 13s\n" % 
		("frame", "node", "X coord", "Y coord", "CPress"))

	for i in range(numberFrames):
		for key in nodeData.keys():
			nd=nodeData[key]
			nX,nY=nodeCoords[key]
			f.write("% 8i % 8i % 12e % 12e % 12e\n" %
				(i,key,nX[i][1],nY[i][1],nd[i][1]))
	f.close()


###
###
###
def reportComponentFromFieldRequest(
	odbFile,
	stepName="",
	instanceName="",
	setName="",
	setType="part",
	field="S",
	fComp=1,
	frames=[],
):
	"""
	The nodal field component (stress, strain) for nodes 
	in set from Field Requests. Nodal values are averaged 
	over elements that share the node of a set.
	""" 
	
	import odbAccess
	import os 

	jobPath,jobOdbName=os.path.split(odbFile)
	jobName=jobOdbName.rstrip(".odb")
	jobDataPath="data/"+jobName+"/"

	myOdb=odbAccess.openOdb(path=odbFile)
	myStep=myOdb.steps[stepName]

	if frames==[]:
		frames=range(1,len(myStep.frames))
		print "(reportComponentFromFieldRequest) Processing ALL frames!"

	if os.path.isdir(jobDataPath)==False:
		os.makedirs(jobDataPath)
	fileName=jobDataPath+jobName+"_"+setName+"-"+stepName+"_"+field+str(fComp)+".fro"

	f=open(fileName,"w")
	print "(reportComponentFromFieldRequest) Writing file ... ", fileName
	f.write("# File automatically generated by reportComponentFromFieldRequest.\n")
	f.write("# Nodal field component "+field+str(fComp)+" for set name "+setName+".\n")
	f.write("# Stress components are in units 1e9Pa.\n")
	f.write("# Data format per line:\n")
	f.write("# % 6s % 8s % 13s % 13s % 13s % 13s\n" % 
		("frame", "node", "X coord", "Y coord", "Z coord", field+str(fComp)))

	for frame in frames:
		print "(reportComponentFromFieldRequest) \tframe ... ", frame
		myFrame=myStep.frames[frame]
		myRegions=myStep.historyRegions
		
		F=myFrame.fieldOutputs[field]
		F_nodal=F.getSubset(
			region=myOdb.rootAssembly.elementSets[" ALL ELEMENTS"],
			position=ELEMENT_NODAL)

		if setType=="root":
			nodes=myOdb.rootAssembly.nodeSets[setName].nodes[0]
			F_set=F_nodal.getSubset(
				region=myOdb.rootAssembly.nodeSets[setName])

		if setType=="part":
			nodes=myOdb.rootAssembly.instances[
				instanceName].nodeSets[setName].nodes
			F_set=F_nodal.getSubset(
				region=myOdb.rootAssembly.instances[
					instanceName].nodeSets[setName])

		FC={}
		for n in nodes:
			sumFC=0.
			count=0.
			for v in F_set.values:
				if v.nodeLabel==n.label:
					count+=1.
					sumFC+=v.data[fComp-1]
			FC[n.label]=sumFC/count
			f.write("% 8i % 8i % 12e % 12e % 12e % 12e\n" %
				(frame,n.label,n.coordinates[0],n.coordinates[1],
				n.coordinates[2],FC[n.label]))
	f.close()

###
###
###
	




